import axios from 'axios'
import { useEffect, useState } from 'react'
import { Link, useLocation, useNavigate } from 'react-router-dom'

import TagSearch from '@/components/TagSearch'
import SectionTitle from '@/components/common/SectionTitle'
import SidebarComponent from '@/components/layout/SidebarComponent'
import { API_BASE_URL } from '@/config'
import { CATEGORIES } from '@/consts'

import type { Post, Tag } from '@/types'

type TagByCategory = { [key: string]: Tag[] }

type Props = { posts: Post[] }


export default ({ posts }: Props) => {
  const navigate = useNavigate ()

  const [tags, setTags] = useState<TagByCategory> ({ })

  const location = useLocation ()
  const query = new URLSearchParams (location.search)
  const tagsQuery = query.get ('tags') ?? ''
  const anyFlg = query.get ('match') === 'any'

  useEffect (() => {
    const tagsTmp: TagByCategory = { }
    let cnt = 0
  loop:
    for (const post of posts)
      {
	for (const tag of post.tags)
	  {
	    if (!(tag.category in tagsTmp))
	      tagsTmp[tag.category] = []
	    if (!(tagsTmp[tag.category].map (t => t.id).includes (tag.id)))
	      {
		tagsTmp[tag.category].push (tag)
		++cnt
		if (cnt >= 25)
		  break loop
	      }
	  }
      }
    for (const cat of Object.keys (tagsTmp))
      tagsTmp[cat].sort ((tagA, tagB) => tagA.name < tagB.name ? -1 : 1)
    setTags (tagsTmp)
  }, [posts])

  return (
      <SidebarComponent>
	<TagSearch />
	<SectionTitle>タグ</SectionTitle>
	<ul>
	  {CATEGORIES.flatMap (cat => cat in tags ? (
	      tags[cat].map (tag => (
		  <li key={tag.id} className="mb-1">
		    <Link to={`/posts?${ (new URLSearchParams ({ tags: tag.name })).toString () }`}>
		      {tag.name}
		    </Link>
		    <span className="ml-1">{tag.postCount}</span>
		  </li>))) : [])}
	</ul>
	<SectionTitle>関聯</SectionTitle>
	{posts.length && (
	    <a href="#"
	       onClick={ev => {
		 ev.preventDefault ()
		 void ((async () => {
		   try
		   {
		     const { data } = await axios.get (`${ API_BASE_URL }/posts/random`,
			 { params: { tags:  tagsQuery.split (' ').filter (e => e !== '').join (','),
				     match: (anyFlg ? 'any' : 'all') } })
		     navigate (`/posts/${ (data as Post).id }`)
		   }
		   catch
		   {
		     ;
		   }
		 }) ())
	       }}>
	      ランダム
	    </a>)}
      </SidebarComponent>)
}
